@@ -78,8 +78,13 @@ pub fn get_instruction_descriptions() -> Vec<SubProgram<'static>> {
             r#else: Default::default(),
         }),
         get_desc(Card::CreateTable),
-        get_desc(Card::SetProperty(Default::default())),
-        get_desc(Card::GetProperty(Default::default())),
+        get_desc(Card::SetProperty),
+        get_desc(Card::GetProperty),
+        get_desc(Card::Len),
+        get_desc(Card::ForEach {
+            lane: Default::default(),
+            variable: Default::default(),
+        }),
     ]
 }
 
@@ -87,21 +92,22 @@ pub fn get_instruction_descriptions() -> Vec<SubProgram<'static>> {
 fn get_desc(node: Card) -> SubProgram<'static> {
     match node {
         Card::CallNative(_)    => unreachable!(),
-        Card::GetProperty(_) => subprogram_description!(
+        Card::GetProperty => subprogram_description!(
             "GetProperty",
             "Gets a named field in the given table. Returns `nil` if the field does not exist",
             SubProgramType::Object,
-            [PropertyName::Object.to_str()],
+            [PropertyName::Object.to_str(), PropertyName::Variable.to_str()],
             [PropertyName::Value.to_str()],
-            [PropertyName::Variable.to_str()]
+            []
         ),
-        Card::SetProperty(_) => subprogram_description!(
+        Card::SetProperty => subprogram_description!(
             "SetProperty",
-            "Sets a named field in the given table to the input value",
+            r#"Sets a named field in the given table to the input value
+Order of parameters: Table, Property-Key, Value"#,
             SubProgramType::Object,
-            [PropertyName::Value.to_str(), PropertyName::Object.to_str()],
+            [PropertyName::Object.to_str(), PropertyName::Variable.to_str(), PropertyName::Value.to_str()],
             [],
-            [PropertyName::Variable.to_str()]
+            []
         ),
         Card::CreateTable => subprogram_description!(
             "CreateTable",
@@ -382,5 +388,23 @@ fn get_desc(node: Card) -> SubProgram<'static> {
             [],
             [PropertyName::Text.to_str()]
         ),
+
+        Card::ForEach {..} => subprogram_description!(
+            "ForEach",
+            "Repeat a lane for each key in the given table, passing the key to the lane as an argument",
+            SubProgramType::Branch,
+            [],
+            [],
+            [PropertyName::Text.to_str(), PropertyName::Text.to_str()]
+        ),
+
+        Card::Len => subprogram_description!(
+            "Len",
+            "Pushes the length of a table (number of keys) onto the stack",
+            SubProgramType::Branch,
+            [PropertyName::Object.to_str()],
+            [PropertyName::Number.to_str()],
+            []
+        ),
     }
 }
