@@ -11,6 +11,7 @@ use self::runtime::CallFrame;
 use crate::{
     binary_compare,
     collections::key_map::{Key, KeyMap},
+    instruction::instruction_span,
     instruction::Instruction,
     pop_stack,
     prelude::*,
@@ -154,6 +155,7 @@ impl<'a, Aux> Vm<'a, Aux> {
     }
 
     /// Initializes a new FieldTable in this VM instance
+    #[inline]
     pub fn init_table(&mut self) -> Result<std::ptr::NonNull<FieldTable>, ExecutionError> {
         self.runtime_data.init_table()
     }
@@ -190,24 +192,50 @@ impl<'a, Aux> Vm<'a, Aux> {
                     self.stack_push(Value::Object(res.as_ptr()))?;
                 }
                 Instruction::GetProperty => {
-                    let handle: Key =
-                        unsafe { instr_execution::decode_value(&program.bytecode, &mut instr_ptr) };
+                    let handle = self.pop_key()?;
                     let instance = self.stack_pop();
                     let table = self.get_table(instance)?;
-                    let result = table.get(handle).copied().unwrap_or(Value::Nil);
+                    let result = table.get_value(handle).unwrap_or(Value::Nil);
                     self.stack_push(result)?;
                 }
                 Instruction::SetProperty => {
-                    let handle: Key =
-                        unsafe { instr_execution::decode_value(&program.bytecode, &mut instr_ptr) };
-                    let instance = self.stack_pop();
                     let value = self.stack_pop();
+                    let key = self.stack_pop();
+                    let instance = self.stack_pop();
                     let table = self.get_table_mut(instance)?;
-                    table.insert(handle, value).map_err(|err| {
+                    table.insert(key, value, self).map_err(|err| {
                         debug!("Failed to insert value {:?}", err);
                         ExecutionError::OutOfMemory
                     })?;
                 }
+                Instruction::BeginRepeat => {
+                    instr_execution::begin_repeat(self)?;
+                }
+                Instruction::Repeat => {
+                    if instr_execution::repeat(self)? >= 0 {
+                        instr_execution::instr_jump(
+                            &mut instr_ptr,
+                            program,
+                            &mut self.runtime_data,
+                        )?;
+                    } else {
+                        instr_ptr += instruction_span(Instruction::CallLane) as usize - 1;
+                    }
+                }
+                Instruction::BeginForEach => {
+                    instr_execution::begin_for_each(self)?;
+                }
+                Instruction::ForEach => {
+                    if instr_execution::for_each(self)? {
+                        instr_execution::instr_jump(
+                            &mut instr_ptr,
+                            program,
+                            &mut self.runtime_data,
+                        )?;
+                    } else {
+                        instr_ptr += instruction_span(Instruction::CallLane) as usize - 1;
+                    }
+                }
                 Instruction::GotoIfTrue => {
                     let condition = self.runtime_data.stack.pop();
                     let pos: i32 =
@@ -282,11 +310,7 @@ impl<'a, Aux> Vm<'a, Aux> {
                 }
                 Instruction::Exit => return Ok(()),
                 Instruction::CopyLast => {
-                    let val = self.runtime_data.stack.last();
-                    self.runtime_data
-                        .stack
-                        .push(val)
-                        .map_err(|_| ExecutionError::Stackoverflow)?;
+                    instr_execution::instr_copy_last(self)?;
                 }
                 Instruction::Pass => {}
                 Instruction::ScalarInt => {
@@ -333,13 +357,13 @@ impl<'a, Aux> Vm<'a, Aux> {
                 Instruction::Call => {
                     instr_execution::execute_call(self, &mut instr_ptr, &program.bytecode)?
                 }
+                Instruction::Len => instr_execution::instr_len(self)?,
             }
         }
 
         Err(ExecutionError::UnexpectedEndOfInput)
     }
 
-    #[inline]
     fn binary_op<F>(&mut self, op: F) -> Result<(), ExecutionError>
     where
         F: Fn(Value, Value) -> Value,
@@ -353,4 +377,22 @@ impl<'a, Aux> Vm<'a, Aux> {
             .map_err(|_| ExecutionError::Stackoverflow)?;
         Ok(())
     }
+
+    fn pop_key(&mut self) -> Result<Key, ExecutionError> {
+        let handle = self.stack_pop();
+        let handle = match handle {
+            Value::Nil => Key::default(),
+            Value::String(s) => {
+                let s = unsafe {
+                    self.get_str(s).ok_or_else(|| {
+                        ExecutionError::invalid_argument("String not found".to_string())
+                    })?
+                };
+                Key::from_str(s).unwrap()
+            }
+            Value::Integer(i) => Key::from(i),
+            Value::Floating(_) | Value::Object(_) => return Err(ExecutionError::Unhashable),
+        };
+        Ok(handle)
+    }
 }
